@@ -1,4 +1,5 @@
 #!/usr/bin/env python3
+# PYTHON_ARGCOMPLETE_OK
 
 import sys
 import os
@@ -13,9 +14,10 @@ from urllib.request import urlopen, Request
 from urllib.error import HTTPError, URLError
 from termcolor import colored
 import colorama  # Required for Windows
+import argcomplete
 
-__version__ = "1.0.0"
-__client_specification__ = "1.2"
+__version__ = "1.1.0"
+__client_specification__ = "1.4"
 
 REQUEST_HEADERS = {'User-Agent': 'tldr-python-client'}
 PAGES_SOURCE_LOCATION = os.environ.get(
@@ -27,14 +29,6 @@ DOWNLOAD_CACHE_LOCATION = os.environ.get(
     'https://tldr-pages.github.io/assets/tldr.zip'
 )
 
-DEFAULT_LANG = os.environ.get(
-    'TLDR_LANGUAGE',
-    os.environ.get('LANG', None)
-).split('_')[0]
-
-if DEFAULT_LANG == 'C':
-    DEFAULT_LANG = None
-
 USE_CACHE = int(os.environ.get('TLDR_CACHE_ENABLED', '1')) > 0
 MAX_CACHE_AGE = int(os.environ.get('TLDR_CACHE_MAX_AGE', 24))
 
@@ -52,6 +46,29 @@ OS_DIRECTORIES = {
 }
 
 
+def get_language_code(language):
+    language = language.split('.')[0]
+    if language in ['pt_PT', 'pt_BR', 'zh_TW']:
+        return language
+    elif language == "pt":
+        return "pt_PT"
+    return language.split('_')[0]
+
+
+def get_default_language():
+    default_lang = get_language_code(
+        os.environ.get(
+            'LANG',
+            'C'
+        )
+    )
+
+    if default_lang == 'C' or default_lang == 'POSIX':
+        default_lang = None
+
+    return default_lang
+
+
 def get_cache_dir():
     if not os.environ.get('XDG_CACHE_HOME', False):
         if not os.environ.get('HOME', False):
@@ -157,19 +174,20 @@ def get_platform_list():
 
 
 def get_language_list():
-    languages = os.environ.get('LANGUAGES', '').split(':')
+    languages = os.environ.get('LANGUAGE', '').split(':')
     languages = list(map(
-        lambda x: x.split('_')[0],
-        filter(lambda x: x != 'C', languages)
+        get_language_code,
+        filter(lambda x: not (x == 'C' or x == 'POSIX' or x == ''), languages)
     ))
-    if DEFAULT_LANG is not None:
-        try:
-            languages.remove(DEFAULT_LANG)
-        except ValueError:
-            pass
-        languages.insert(0, DEFAULT_LANG)
+
+    default_lang = get_default_language()
+
+    if default_lang is not None and default_lang not in languages:
+        languages.append(default_lang)
     else:
-        languages.append('en')
+        languages = []
+    if 'en' not in languages:
+        languages.append(None)
     return languages
 
 
@@ -223,6 +241,20 @@ COMMAND_SPLIT_REGEX = re.compile(r'(?P<param>{{.+?}})')
 PARAM_REGEX = re.compile(r'(?:{{)(?P<param>.+?)(?:}})')
 
 
+def get_commands(platforms=None):
+    if platforms is None:
+        platforms = get_platform_list()
+
+    commands = []
+    if os.path.exists(get_cache_dir()):
+        for platform in platforms:
+            path = os.path.join(get_cache_dir(), 'pages', platform)
+            if not os.path.exists(path):
+                continue
+            commands += [file[:-3] for file in os.listdir(path) if file.endswith(".md")]
+    return commands
+
+
 def colors_of(key):
     env_key = 'TLDR_COLOR_%s' % key.upper()
     values = os.environ.get(env_key, DEFAULT_COLORS[key]).strip().split()
@@ -256,7 +288,7 @@ def output(page):
                     colored(
                         line.replace('>', '').replace('<', ''),
                         *colors_of('description')
-                    )
+                )
                 sys.stdout.buffer.write(line.encode('utf-8'))
             elif line[0] == '-':
                 line = '\n' + ' ' * LEADING_SPACES_NUM + \
@@ -280,7 +312,8 @@ def output(page):
 
 def update_cache(language=None):
     if language is None:
-        language = DEFAULT_LANG if DEFAULT_LANG is not None else 'en'
+        default_lang = get_default_language()
+        language = default_lang if default_lang is not None else 'en'
     elif isinstance(language, list):
         language = language[0]
     try:
@@ -312,8 +345,8 @@ def update_cache(language=None):
 def main():
     parser = ArgumentParser(
         prog="tldr",
-        usage="tldr [-u] [-p PLATFORM] [-s SOURCE] [-c] [-r] [-L LANGUAGE] " +
-        "command",
+        usage="tldr [-u] [-p PLATFORM] [-l] [-s SOURCE] [-c] [-r] [-L LANGUAGE]" +
+        " command",
         description="Python command line client for tldr"
     )
     parser.add_argument(
@@ -339,6 +372,11 @@ def main():
         help="Override the operating system [linux, osx, sunos, windows, common]"
     )
 
+    parser.add_argument('-l', '--list',
+                        default=False,
+                        action='store_true',
+                        help="List all available commands for operating system")
+
     parser.add_argument('-s', '--source',
                         default=PAGES_SOURCE_LOCATION,
                         type=str,
@@ -363,9 +401,10 @@ def main():
                         help='Override the default language')
 
     parser.add_argument(
-        'command', type=str, nargs='*', help="command to lookup"
-    )
+        'command', type=str, nargs='*', help="command to lookup", metavar='command'
+    ).completer = argcomplete.completers.ChoicesCompleter(get_commands())
 
+    argcomplete.autocomplete(parser)
     options = parser.parse_args()
 
     colorama.init(strip=options.color)
@@ -377,7 +416,9 @@ def main():
         parser.print_help(sys.stderr)
         sys.exit(1)
 
-    if options.render:
+    if options.list:
+        print(get_commands(options.platform))
+    elif options.render:
         for command in options.command:
             if os.path.exists(command):
                 with open(command, encoding='utf-8') as open_file:
@@ -392,11 +433,11 @@ def main():
                 options.language
             )
             if not result:
-                print((
+                sys.exit((
                     "`{cmd}` documentation is not available. "
                     "Consider contributing Pull Request to "
                     "https://github.com/tldr-pages/tldr"
-                ).format(cmd=command), file=sys.stderr)
+                ).format(cmd=command))
             else:
                 output(result)
         except URLError as e:
